/*
stattest - tests for randomness
Copyright (C) 2003  Roland Siegert, Andreas Stricker

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
*/

/** Sample autocorrelation test
 */
 
#include <stdio.h>
#include <stdlib.h>

void printbin(unsigned long b) {
	int i;
	for (i = 31; i >= 0; i--) printf("%d", (b >> i) & 0x01);
}

unsigned long rotbyte(unsigned long b) {
	return (b >> 1) | ((b << 31) & 0x80000000);
}

double autocorr(unsigned long b1, unsigned long b2) {
	int i, n, count;
	int value[32];
	int min, max, sum;
	double res;

	for (i = 0; i < 32; i++) value[i] = 0;

	for (i = 0; i < 32; i++) {
#ifdef DEBUG
		printf("b1:"); printbin(b1); printf("\n");
		printf("b2:"); printbin(b2);
		printf(" ac:"); printbin(b1 ^ b2);
#endif
		count = 0;
		for (n = 0; n < 32; n++) count += (((b1 ^ b2) >> n) & 0x0001);
#ifdef DEBUG
		printf(" v: %2d bits: %d\n", i, count);
#endif
		value[count]++;
		b2 = rotbyte(b2);
	}

	min = 32; max = 0; sum = 0;
	for (i = 0; i < 32; i++) {
		sum += value[i];
		min = ((value[i]) != 0 && (i < min)) ? i : min;
		max = ((value[i]) != 0 && (i > max)) ? i : max;
	}
	res = 0.0;
	for (i = 0; i < 32; i++) {
		res += value[i]*((double) max / (double) min);
	}
	res /= sum;
	return res;
}

int main(int args, char *argv[]) {
	int i, j;
	double res;
/*
	unsigned long b[] = {
		0xd37106a2,
		0xa931be07,
		0xd7770317,
		0x265da87f,
		0x33612e59
	};*/
	unsigned long b[] = {
		0x89ABCDEF,
		0xa9d37131,
		0xaaaaaaaa,
		0x0000AB00,
		0x00AD0000
	};
	for (i = 0; i < 5; i++) {
		for (j = 0; j < 5; j++) {
			res = autocorr(b[i], b[j]);
			printf("%8lx\t %8lx\t: %.5f\n", b[i], b[j], res);
		}
	}
} // main()
