/*
stattest - tests for randomness
Copyright (C) 2003  Roland Siegert, Andreas Stricker

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
*/

/** \file
 * \brief Module entropy: Calculate entropy of the buffer
 *
 * Calculate entropy of the buffer.
 *
 * \author Andreas Stricker
 * \author Roland Siegert
 * \version $Id: entropie.c,v 1.5 2003/05/16 14:07:33 stricand Exp $
 */
 
#include <math.h>
 
#include "entropie.h"
#include "stattest.h"
#include "histogram.h"

/** \brief Calculate entropy of buffer.
 * Note that this mean we also need a histogramm even if we don't display it.
 * \param bufsize size of the buffer
 * \param histogram the previous generated histogram
 * \return entropy of the buffer. Should be close to BLOCK_LENGTH.
 * \sa build_histogram()
 */
double calc_entropy(size_t bufsize, histogram_t histogram) {
	double entropy, p_i;
	int i;

	entropy = 0.0;
	for (i = 0; i < BLOCK_RANGE; i++) {
		if (histogram[i] != 0) {
			p_i = ((double) histogram[i]) / bufsize;
			entropy += p_i * (-1.0) * log(p_i) / log(2.0);
		}
	}
	
	return entropy;
} // calc_entropie()
