/*
stattest - tests for randomness
Copyright (C) 2003  Roland Siegert, Andreas Stricker

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
*/

/** \file
 * \brief Module histogram: Build histogram of the buffer
 *
 * Build a histogram of the buffer. One unit is a block.
 *
 * \author Andreas Stricker
 * \author Roland Siegert
 * \version $Id: histogram.c,v 1.3 2003/05/15 07:54:54 stricand Exp $
 */
 
#include <stdio.h>
#include <stdlib.h>

#include "histogram.h"
#include "stattest.h"

/** Build a histogram from the buffer.
 * Note: if parameter histogram points to NULL than new memory is allocated.
 */
histogram_t *build_histogram(buffer_t buffer, size_t bufsize, histogram_t *histogram) {
	int i;
	size_t bufpos;
	
	if (histogram == NULL) {
		histogram = (histogram_t*) malloc(sizeof(histogram_t));
		if (histogram == NULL) return NULL;	// error!
	}
	
	// initialize histogram
	for (i = 0; i < BLOCK_RANGE; i++) (*histogram)[i] = 0;
	
	// build histogram
	for (bufpos = 0; bufpos < bufsize; bufpos++) (*histogram)[buffer[bufpos]]++;
	
	return histogram;
} // build_histogram()

/** Print histogram to screen
 */
void print_histogram(histogram_t histogram) {
	int i, j, barlength;
	unsigned long max, min, average;

	// find maximal and minimal value in histogram
	max = 0; min = ~0; average = 0;
	for (i = 0; i < BLOCK_RANGE; i++) {
		max = (max < histogram[i]) ? histogram[i] : max;
		min = (min > histogram[i]) ? histogram[i] : min;
		average += histogram[i];
	}
	average /= BLOCK_RANGE;		// or easier: buffersize / BLOCK_RANGE

	// print histogram
	for (i = 0; i < BLOCK_RANGE; i++) {
		printf("%2x %8ld ", i, histogram[i]);
		barlength = (HISTOGRAM_WIDTH*histogram[i]) / max;
		for (j = 0; j < barlength; j++) printf(HISTOGRAM_BAR_CHAR);
		printf("\n");
	}
	// print additional information
	printf("Min: %ld Average: %ld Max: %ld\n", min, average, max);

} // print_histogram()
