/*
stattest - tests for randomness
Copyright (C) 2003  Roland Siegert, Andreas Stricker

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
*/

/** \file
 * \brief Main part and user interface for statistic test software
 *
 * This utility test a random source to its randomness.
 * The central test is the "universal statistical test"
 * Some important function like bias, entropy and histogram also
 * included
 *
 * \author Andreas Stricker
 * \author Roland Siegert
 * \version $Id: stattest.h,v 1.12 2003/05/16 14:07:33 stricand Exp $
 */

#ifndef STATTEST_H_8A5D
#define STATTEST_H_8A5D

#include <unistd.h>

#define VERSION "1.0"

#define MSG_START \
	"Statistical test for randomness, Andreas Stricker, Roland Siegert\n"
#define MSG_QUOTE \
	"A sequence is random, if the smallest description of it is the sequence"\
	" itself\n"
#define MSG_ERR_ARG_REJECTRATE_NO_NUM \
	"Rejection rate option need a floating point argument\n"
#define MSG_ERR_ARG_REJECTRATE_OUT_OF_RANGE \
	"Rejection rate out of range ]0.0 - 0.5[\n"
#define MSG_ERR_ARG_BLOCKSIZE_NO_NUM \
	"Blocksize L option need a number argument\n"
#define MSG_ERR_ARG_BLOCKSIZE_OUT_OF_RANGE \
	"Blocksize is not in {8, 16}\n"
#define MSG_ERR_ARG_MAXSIZE_NO_NUM \
	"maxsize option need a number argument\n"
#define MSG_ERR_ARG_MAXSIZE_OUT_OF_RANGE \
	"maxsize out of range [1 - %ul]\n"
#define MSG_ERR_ARG_UNKNOWN \
	"Unknown argument \"%c\"\n"
#define MSG_ERR_ARG_NOFILE \
	"No file specified\n"
#define MSG_ERR_FILE_NOT_ACCESSIBLE \
	"File \"%s\" not accessible, check for existence and permissions\n"
#define MSG_ERR_NO_ENOUGH_MEMORY \
	"No enough memory to allocate the buffer.\n"
#define MSG_ERR_CANT_READ_WHOLE_FILE \
	"Cannot read whole file. %d Bytes still to read.\n"
#define MSG_ERR_CANT_OPEN_FILE \
	"Cannot open file. Please check permissions\n"
#define MSG_INFO_FILE_TOO_BIG \
	"File is bigger than %1$d Bytes. Testing only first %1$d Bytes\n"

#define MSG_VERSION \
	"Version %s\n"

#define MSG_HELP \
"\nsyntax: %s [OPTION] [-h] FILE\n"\
"PARAMETERS:\n"\
"        FILE                 File to test for randomness\n"\
"OPTIONS:\n"\
"      --entropy    -e       Only return entropy\n"\
"      --bias       -b       Only return bias\n"\
"      --histogram  -i       Only return histogram\n"\
"      --universal  -u       Only return universal statistical test\n"\
"      --rejectrate RATE     Set rejection rate to RATE used by\n"\
"                   -r RATE   universal statistical test (Default 0.001)\n"\
"      --blocksize  -L       Set blocksize L for universal statistical test\n"\
"      --fixedq     -Q       Set initialisation length to fixed size: 10*2^L\n"\
"      --maxsize SIZE        Only read first SIZE Bytes of file. It possible\n"\
"                   -m SIZE   to use 'k' for kilobytes and 'M' for megabytes\n"\
"      --help       -h       Show this help\n"\
"      --version    -V       Print version and exit\n"\
"\n"

// I need a boolean
#define true 1
#define false 0
typedef char bool;

// structure saving settings
struct settings_struct {
	bool is_entropy;
	bool is_bias;
	bool is_histogram;
	bool is_universal;
	double rejection_rate;
	int blocksize_L;
	bool fixed_Q;
	size_t maxsize;
	char *filename;
};

// define the type of buffer, used to test
#define BLOCK_LENGTH 8				// Size of a block in bits
#define BLOCK_RANGE (0x01 << BLOCK_LENGTH)	// range of a block (2^BLOCK_LENGTH)
typedef unsigned char block_t;		// data type of a block
typedef block_t *buffer_t;			// pointer to an array of blocks

#define MAX_BUF_SIZE 16*1024*1024	// set maximal size of a block to 16MByte

#define DEFAULT_REJECTION_RATE 0.01	// default rejection rate for UST
#define DEFAULT_BLOCKSIZE_L 8		// default block size L for UST
#define DEFAULT_MAX_SIZE 0			// default maxsize value (0 = MAX_BUF_SIZE)

typedef unsigned char UINT8;
typedef unsigned short int UINT16;
typedef unsigned int UINT32;

#endif
